using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of AuthAPI domain.
    /// </summary>
    public sealed class AuthAPIModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public AuthAPIModule(AuthAPIService service) : base("/api")
        { 
            Post["/auth/login"] = parameters =>
            {
                var login = Parameters.ValueOf<string>(parameters, Context.Request, "login", ParameterType.Query);
                var salt = Parameters.ValueOf<string>(parameters, Context.Request, "salt", ParameterType.Query);
                var response = Parameters.ValueOf<string>(parameters, Context.Request, "response", ParameterType.Query);
                Preconditions.IsNotNull(login, "Required parameter: 'login' is missing at 'AuthLoginPost'");
                
                Preconditions.IsNotNull(salt, "Required parameter: 'salt' is missing at 'AuthLoginPost'");
                
                Preconditions.IsNotNull(response, "Required parameter: 'response' is missing at 'AuthLoginPost'");
                
                return service.AuthLoginPost(Context, login, salt, response);
            };

            Get["/auth/start"] = parameters =>
            {
                var login = Parameters.ValueOf<string>(parameters, Context.Request, "login", ParameterType.Query);
                Preconditions.IsNotNull(login, "Required parameter: 'login' is missing at 'AuthStartGet'");
                
                return service.AuthStartGet(Context, login);
            };
        }
    }

    /// <summary>
    /// Service handling AuthAPI requests.
    /// </summary>
    public interface AuthAPIService
    {
        /// <summary>
        /// Возвращает ключ доступа, с которым в дальнейшем будут осуществляться всё взаимодействие с API  (параметр &#39;token&#39;). Для вычисления значения параметра &#39;response&#39; необходимо получить представленную в hex MD5-сумму от  объединения строк пароля пользователя, полученной соли и символа переноса строки. Псевдокод операции:  hex(md5(password&amp;nbsp;+&amp;nbsp;salt&amp;nbsp;+&amp;nbsp;&#39;\\n&#39;))
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="login">Имя пользователя</param>
        /// <param name="salt">Соль (см. /auth/start)</param>
        /// <param name="response">Вычисленный хэш</param>
        /// <returns>string</returns>
        string AuthLoginPost(NancyContext context, string login, string salt, string response);

        /// <summary>
        /// Возвращает соль для авторизации. Это первый этап идентификации пользователя. Затем соль используется  во втором, окончательном этапе авторизации, в запросе /auth/login. Запрос не требует дополнительных параметров.
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="login">Имя пользователя</param>
        /// <returns>string</returns>
        string AuthStartGet(NancyContext context, string login);
    }

    /// <summary>
    /// Abstraction of AuthAPIService.
    /// </summary>
    public abstract class AbstractAuthAPIService: AuthAPIService
    {
        public virtual string AuthLoginPost(NancyContext context, string login, string salt, string response)
        {
            return AuthLoginPost(login, salt, response);
        }

        public virtual string AuthStartGet(NancyContext context, string login)
        {
            return AuthStartGet(login);
        }

        protected abstract string AuthLoginPost(string login, string salt, string response);

        protected abstract string AuthStartGet(string login);
    }

}
